@@ -2,6 +2,8 @@
  * Routines for calling the right protocol for the ethertype.
  * This is called by both packet-eth.c (Ethernet II) and packet-llc.c (SNAP)
  *
+ * $Id: ethertype.c,v 1.5 1998/10/10 03:32:06 gerald Exp $
+ *
  * Gilbert Ramirez <gram@verdict.uthscsa.edu>
  *
  * Ethereal - Network traffic analyzer
@@ -24,14 +26,22 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#ifdef HAVE_SYS_TYPES_H
+# include <sys/types.h>
+#endif
+
 #include <gtk/gtk.h>
 
 #include <stdio.h>
 
 #include <pcap.h>
 
-#include "packet.h"
 #include "ethereal.h"
+#include "packet.h"
 #include "etypes.h"
 
 void
@@ -49,6 +59,13 @@ ethertype(guint16 etype, int offset,
       }
       dissect_ip(pd, offset, fd, tree);
       break;
+    case ETHERTYPE_IPv6:
+      if (tree) {
+        add_item_to_tree(fh_tree, offset - 2, 2, "Type: IPv6 (0x%04x)",
+          etype);
+      }
+      dissect_ipv6(pd, offset, fd, tree);
+      break;
     case ETHERTYPE_ARP:
       if (tree) {
         add_item_to_tree(fh_tree, offset - 2, 2,
@@ -68,14 +85,14 @@ ethertype(guint16 etype, int offset,
         add_item_to_tree(fh_tree, offset - 2, 2,
           "Type: AppleTalk (0x%04x)", etype);
       }
-      if (fd->win_info[0]) { strcpy(fd->win_info[3], etype_str[3]); }
+      if (fd->win_info[COL_NUM]) { strcpy(fd->win_info[COL_PROTOCOL], etype_str[3]); }
       break;
     case ETHERTYPE_AARP:
       if (tree) {
         add_item_to_tree(fh_tree, offset - 2, 2,
           "Type: AARP (0x%04x)", etype);
       }
-      if (fd->win_info[0]) { strcpy(fd->win_info[3], etype_str[4]); }
+      if (fd->win_info[COL_NUM]) { strcpy(fd->win_info[COL_PROTOCOL], etype_str[4]); }
       break;
     case ETHERTYPE_IPX:
       if (tree) {
@@ -84,13 +101,20 @@ ethertype(guint16 etype, int offset,
       }
       dissect_ipx(pd, offset, fd, tree);
       break;
+    case ETHERTYPE_VINES:
+      if (tree) {
+        add_item_to_tree(fh_tree, offset - 2, 2,
+          "Type Vines (0x%04x)", etype);
+      }
+      dissect_vines(pd, offset, fd, tree);
+      break;
     default:
       if (tree) {
         add_item_to_tree(fh_tree, offset - 2, 2,
           "Type: Unknown (0x%04x)", etype);
-		  dissect_raw(pd, offset, fd, tree);
+		  dissect_data(pd, offset, fd, tree);
 	  }
-      if (fd->win_info[0]) { sprintf(fd->win_info[3], "0x%04x", etype); }
+      if (fd->win_info[COL_NUM]) { sprintf(fd->win_info[COL_PROTOCOL], "0x%04x", etype); }
       break;
   }
  }
