@@ -1,6 +1,8 @@
 /* file.h
  * Definitions for file structures and routines
  *
+ * $Id: file.h,v 1.4 1998/10/12 01:40:49 gerald Exp $
+ *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
  * Copyright 1998 Gerald Combs
@@ -46,16 +48,13 @@
 #define SNOOP_MIN_VERSION 2
 #define SNOOP_MAX_VERSION 2
 
-/* Link types */
-#define LT_UNKNOWN 0
-#define LT_ETHER10 1
-#define LT_IEEE802 6
-#define LT_PPP     12
+/* Link types (removed in favor of the DLT_* defines from bpf.h */
 
 typedef struct bpf_program bpf_prog;
 
 typedef struct _capture_file {
   FILE       *fh;        /* Capture file */
+  gchar      *filename;  /* filename */
   long        f_len;     /* File length */
   int         swap;      /* Swap data bytes? */
   guint16     cd_t;      /* Capture data type */
@@ -69,7 +68,8 @@ typedef struct _capture_file {
   gchar      *iface;     /* Interface */
   gchar      *save_file; /* File to write capture data */
   pcap_t     *pfh;       /* Pcap session */
-  gchar      *filter;    /* Pcap filter string */
+  gchar      *dfilter;   /* Display filter string */
+  gchar      *cfilter;   /* Capture filter string */
   bpf_prog    fcode;     /* Compiled filter program */
   guint8      pd[4096];  /* Packet data */
   GList      *plist;     /* Packet list */
