@@ -2,6 +2,8 @@
  * Routines for IEEE 802.2 LLC layer
  * Gilbert Ramirez <gram@verdict.uthscsa.edu>
  *
+ * $Id: packet-llc.c,v 1.8 1998/10/10 03:32:12 gerald Exp $
+ *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@unicom.net>
  * Copyright 1998 Gerald Combs
@@ -22,52 +24,63 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#ifdef HAVE_SYS_TYPES_H
+# include <sys/types.h>
+#endif
+
 #include <pcap.h>
 
 #include <gtk/gtk.h>
 
 #include <stdio.h>
 
-#include "packet.h"
 #include "ethereal.h"
+#include "packet.h"
 #include "etypes.h"
 
 struct sap_info {
-	u_char	sap;
+	guint8	sap;
+	void	(*func) (const u_char *, int, frame_data *, GtkTree *);
 	char	*text;
 };
 
+static struct sap_info	saps[] = {
+	{ 0x00, NULL,		"NULL LSAP" },
+	{ 0x02, NULL,		"LLC Sub-Layer Management Individual" },
+	{ 0x03, NULL,		"LLC Sub-Layer Management Group" },
+	{ 0x04, NULL,		"SNA Path Control Individual" },
+	{ 0x05, NULL,		"SNA Path Control Group" },
+	{ 0x06, dissect_ip,	"TCP/IP" },
+	{ 0x08, NULL,		"SNA" },
+	{ 0x0C, NULL,		"SNA" },
+	{ 0x42, NULL,		"Spanning Tree BPDU" },
+	{ 0x7F, NULL,		"ISO 802.2" },
+	{ 0x80, NULL,		"XNS" },
+	{ 0xAA, NULL,		"SNAP" },
+	/*{ 0xBA, dissect_vines,	"Banyan Vines" },
+	{ 0xBC, dissect_vines,	"Banyan Vines" },*/
+	{ 0xBA, NULL,		"Banyan Vines" },
+	{ 0xBC, NULL,		"Banyan Vines" },
+	{ 0xE0, dissect_ipx,	"NetWare" },
+	{ 0xF0, NULL,		"NetBIOS" },
+	{ 0xF4, NULL,		"IBM Net Management Individual" },
+	{ 0xF5, NULL,		"IBM Net Management Group" },
+	{ 0xF8, NULL,		"Remote Program Load" },
+	{ 0xFC, NULL,		"Remote Program Load" },
+	{ 0xFE, dissect_osi,	"ISO Network Layer" },
+	{ 0xFF, NULL,		"Global LSAP" },
+	{ 0x00, NULL,		NULL }
+};
+
+
 static char*
 sap_text(u_char sap) {
 	int i=0;
 
-	static struct sap_info	saps[] = {
-		{ 0x00, "NULL LSAP" },
-		{ 0x02, "LLC Sub-Layer Management Individual" },
-		{ 0x03, "LLC Sub-Layer Management Group" },
-		{ 0x04, "SNA Path Control Individual" },
-		{ 0x05, "SNA Path Control Group" },
-		{ 0x06, "TCP/IP" },
-		{ 0x08, "SNA" },
-		{ 0x0C, "SNA" },
-		{ 0x42, "Spanning Tree BPDU" },
-		{ 0x7F, "ISO 802.2" },
-		{ 0x80, "XNS" },
-		{ 0xAA, "SNAP" },
-		{ 0xBA, "Banyan Vines" },
-		{ 0xBC, "Banyan Vines" },
-		{ 0xE0, "NetWare" },
-		{ 0xF0, "NetBIOS" },
-		{ 0xF4, "IBM Net Management Individual" },
-		{ 0xF5, "IBM Net Management Group" },
-		{ 0xF8, "Remote Program Load" },
-		{ 0xFC, "Remote Program Load" },
-		{ 0xFE, "ISO Network Layer" },
-		{ 0xFF, "Global LSAP" },
-		{ 0x00, NULL }
-	};
-
 	while (saps[i].text != NULL) {
 		if (saps[i].sap == sap) {
 			return saps[i].text;
@@ -77,6 +90,19 @@ sap_text(u_char sap) {
 	return "Unknown";
 }
 
+static void*
+sap_func(u_char sap) {
+	int i=0;
+
+	while (saps[i].text != NULL) {
+		if (saps[i].sap == sap) {
+			return saps[i].func;
+		}
+		i++;
+	}
+	return dissect_data;
+}
+
 static char*
 llc_org(const u_char *ptr) {
 
@@ -95,9 +121,10 @@ llc_org(const u_char *ptr) {
 void
 dissect_llc(const u_char *pd, int offset, frame_data *fd, GtkTree *tree) {
 
-	GtkWidget	*llc_tree, *ti;
+	GtkWidget	*llc_tree = NULL, *ti;
 	guint16		etype;
-	int			is_snap;
+	int		is_snap;
+	void		(*dissect) (const u_char *, int, frame_data *, GtkTree *);
 
 	/* LLC Strings */
 	char *llc_ctrl[4] = {
@@ -106,15 +133,15 @@ dissect_llc(const u_char *pd, int offset, frame_data *fd, GtkTree *tree) {
 
 	is_snap = (pd[offset] == 0xAA) && (pd[offset+1] == 0xAA);
 
-	if (fd->win_info[0]) {
-		strcpy(fd->win_info[3], "LLC");
+	if (fd->win_info[COL_NUM]) {
+		strcpy(fd->win_info[COL_PROTOCOL], "LLC");
 	}
   
 	if (tree) {
 		ti = add_item_to_tree(GTK_WIDGET(tree), offset, (is_snap ? 8 : 3),
 			"Logical-Link Control");
 		llc_tree = gtk_tree_new();
-		gtk_tree_item_set_subtree(GTK_TREE_ITEM(ti), llc_tree);
+		add_subtree(ti, llc_tree, ETT_LLC);
 		add_item_to_tree(llc_tree, offset,      1, "DSAP: %s (0x%02X)",
 			sap_text(pd[offset]), pd[offset]);
 		add_item_to_tree(llc_tree, offset+1,    1, "SSAP: %s (0x%02X)",
@@ -124,8 +151,8 @@ dissect_llc(const u_char *pd, int offset, frame_data *fd, GtkTree *tree) {
 	}
 
 	if (is_snap) {
-		if (fd->win_info[0]) {
-			strcpy(fd->win_info[4], "802.2 LLC (SNAP)");
+		if (fd->win_info[COL_NUM]) {
+			strcpy(fd->win_info[COL_INFO], "802.2 LLC (SNAP)");
 		}
 		if (tree) {
 			add_item_to_tree(llc_tree, offset+3,    3,
@@ -138,23 +165,21 @@ dissect_llc(const u_char *pd, int offset, frame_data *fd, GtkTree *tree) {
 		ethertype(etype, offset, pd, fd, tree, llc_tree);
 	}		
 	else {
-		if (fd->win_info[0]) {
-			sprintf(fd->win_info[4], "802.2 LLC (%s)", sap_text(pd[offset]));
+		if (fd->win_info[COL_NUM]) {
+			sprintf(fd->win_info[COL_INFO], "802.2 LLC (%s)", sap_text(pd[offset]));
 		}
 
+		dissect = sap_func(pd[offset]);
+
 		/* non-SNAP */
 		offset += 3;
 
-		switch (pd[offset-3]) {
-			case 0x06:	/* TCP/IP */
-				dissect_ip(pd, offset, fd, tree);
-				break;
-			case 0xe0:	/* NetWare (IPX) */
-				dissect_ipx(pd, offset, fd, tree);
-				break;
-			default:
-				dissect_raw(pd, offset, fd, tree);
-				break;
-		  }
+		if (dissect) {
+			dissect(pd, offset, fd, tree);
+		}
+		else {
+			dissect_data(pd, offset, fd, tree);
+		}
+
 	}
 }
