@@ -1,6 +1,8 @@
 /* packet-tcp.c
  * Routines for TCP packet disassembly
  *
+ * $Id: packet-tcp.c,v 1.4 1998/09/27 22:12:38 gerald Exp $
+ *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
  * Copyright 1998 Gerald Combs
@@ -21,22 +23,28 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <gtk/gtk.h>
 
 #include <stdio.h>
 
 #ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
+# include <sys/types.h>
 #endif
 
 #ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
+# include <netinet/in.h>
 #endif
 
 #include "ethereal.h"
 #include "packet.h"
+#include "follow.h"
+
+extern FILE* data_out_file;
+extern packet_info pi;
 
 void
 dissect_tcp(const u_char *pd, int offset, frame_data *fd, GtkTree *tree) {
@@ -71,9 +79,9 @@ dissect_tcp(const u_char *pd, int offset, frame_data *fd, GtkTree *tree) {
   }
   flags[fpos] = '\0';
   
-  if (fd->win_info[0]) {
-    strcpy(fd->win_info[3], "TCP");
-    sprintf(fd->win_info[4], "Source port: %d  Destination port: %d",
+  if (fd->win_info[COL_NUM]) {
+    strcpy(fd->win_info[COL_PROTOCOL], "TCP");
+    sprintf(fd->win_info[COL_INFO], "Source port: %d  Destination port: %d",
       th.th_sport, th.th_dport);
   }
   
@@ -81,7 +89,7 @@ dissect_tcp(const u_char *pd, int offset, frame_data *fd, GtkTree *tree) {
     ti = add_item_to_tree(GTK_WIDGET(tree), offset, 20,
       "Transmission Control Protocol");
     tcp_tree = gtk_tree_new();
-    gtk_tree_item_set_subtree(GTK_TREE_ITEM(ti), tcp_tree);
+    add_subtree(ti, tcp_tree, ETT_TCP);
     add_item_to_tree(tcp_tree, offset,      2, "Source port: %d", th.th_sport);
     add_item_to_tree(tcp_tree, offset +  2, 2, "Destination port: %d", th.th_dport);
     add_item_to_tree(tcp_tree, offset +  4, 4, "Sequence number: 0x%08x",
@@ -96,8 +104,32 @@ dissect_tcp(const u_char *pd, int offset, frame_data *fd, GtkTree *tree) {
       th.th_urp);
     /* To do: TCP options */
 
-    /* Skip over header + options */
-    offset += 4 * th.th_off;
-    dissect_raw(pd, offset, fd, tree);
   }
+    /* Skip over header + options */
+	offset += 4 * th.th_off;
+
+	/* until we decode those options, I'll check the packet length
+	to see if there's more data. -- gilbert */
+	if (fd->cap_len > offset) {
+		switch(MIN(th.th_sport, th.th_dport)) {
+			case TCP_PORT_PRINTER:
+				dissect_lpd(pd, offset, fd, tree);
+				break;
+			default:
+				dissect_data(pd, offset, fd, tree);
+		}
+	}
+ 
+	pi.srcport = th.th_sport;
+	pi.destport = th.th_dport;
+	
+	if( data_out_file ) {
+	  reassemble_tcp( th.th_seq, /* sequence number */
+			  ( pi.iplen -( pi.iphdrlen * 4 )-( th.th_off * 4 ) ), /* length */
+			  ( pd+offset ), /* data */
+			  ( th.th_flags & 0x02 ), /* is syn set? */
+			  pi.ip_src ); /* src ip */
+	}
+
+
 }
