@@ -2,6 +2,8 @@
  * Routines for Token-Ring packet disassembly
  * Gilbert Ramirez <gram@verdict.uthscsa.edu>
  *
+ * $Id: packet-tr.c,v 1.5 1998/09/27 22:12:39 gerald Exp $
+ *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@unicom.net>
  * Copyright 1998 Gerald Combs
@@ -22,22 +24,26 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include <gtk/gtk.h>
 
 #include <stdio.h>
 
 #ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
+# include <sys/types.h>
 #endif
 
 #ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
+# include <netinet/in.h>
 #endif
 
 #include <pcap.h>
 
-#include "packet.h"
 #include "ethereal.h"
+#include "packet.h"
 #include "etypes.h"
 
 static void
@@ -68,18 +74,29 @@ sr_frame(u_char val) {
 	else return rc_frame[val];
 }
 
+
 void
 dissect_tr(const u_char *pd, frame_data *fd, GtkTree *tree) {
 
 	GtkWidget	*fh_tree, *ti;
 	int			offset = 14;
+
 	int			source_routed = 0;
-	int			rif_bytes = 0;
-	guint8		nonsr_hwaddr[8];
-	int			frame_type = (pd[1] & 192) >> 6; /* I use this value a lot */
-	#ifdef linux
-	int			silly_linux = 0;
-	#endif
+	int			frame_type;
+	guint8			trn_rif_bytes;
+	guint8			actual_rif_bytes;
+
+	/* The trn_hdr struct, as separate variables */
+	guint8			trn_ac;		/* access control field */
+	guint8			trn_fc;		/* field control field */
+	guint8			trn_dhost[6];	/* destination host */
+	guint8			trn_shost[6];	/* source host */
+	guint16			trn_rcf;	/* routing control field */
+	guint16			trn_rseg[8];	/* routing registers */
+
+	/* non-source-routed version of source addr */
+	guint8			trn_shost_nonsr[6];
+
 
 	/* Token-Ring Strings */
 	char *fc[] = { "MAC", "LLC", "Reserved" };
@@ -91,64 +108,110 @@ dissect_tr(const u_char *pd, frame_data *fd, GtkTree *tree) {
 	char *rc_direction[] = { "From originating station",
 		"To originating station" };
 
+	/* get the data */
+	memcpy(&trn_ac, &pd[0], sizeof(guint8));
+	memcpy(&trn_fc, &pd[1], sizeof(guint8));
+	memcpy(trn_dhost, &pd[2], 6 * sizeof(guint8));
+	memcpy(trn_shost, &pd[8], 6 * sizeof(guint8));
+	memcpy(&trn_rcf, &pd[14], sizeof(guint16));
+	memcpy(trn_rseg, &pd[16], 8 * sizeof(guint16));
+
+	memcpy(trn_shost_nonsr, &pd[8], 6 * sizeof(guint8));
+	trn_shost_nonsr[0] &= 127;
+	frame_type = (trn_fc & 192) >> 6;
+
 	/* if the high bit on the first byte of src hwaddr is 1, then
 		this packet is source-routed */
-	source_routed = pd[8] & 128;
-	if (source_routed) {
-		rif_bytes = pd[14] & 31;
+	source_routed = trn_shost[0] & 128;
+
+	trn_rif_bytes = pd[14] & 31;
+
+	/* sometimes we have a RCF but no RIF... half source-routed? */
+	/* I'll check for 2 bytes of RIF and the 0x70 byte */
+	if (!source_routed) {
+		if (trn_rif_bytes == 2) {
+			source_routed = 1;
+		}
+		/* the Linux 2.0 TR code strips source-route bits in
+		 * order to test for SR. This can be removed from most
+		 * packets with oltr, but not all. So, I try to figure out
+		 * which packets should have been SR here. I'll check to
+		 * see if there's a SNAP or IPX field right after
+		 * my RIF fields.
+		 */
+		else if ( (
+			pd[0x0e + trn_rif_bytes] == 0xaa &&
+			pd[0x0f + trn_rif_bytes] == 0xaa &&
+			pd[0x10 + trn_rif_bytes] == 0x03) ||
+			  (
+			pd[0x0e + trn_rif_bytes] == 0xe0 &&
+			pd[0x0f + trn_rif_bytes] == 0xe0) ) {
+
+			source_routed = 1;
+		}
+/*		else {
+			printf("0e+%d = %02X   0f+%d = %02X\n", trn_rif_bytes, pd[0x0e + trn_rif_bytes],
+					trn_rif_bytes, pd[0x0f + trn_rif_bytes]);
+		} */
+
 	}
+
+	if (source_routed) {
+		actual_rif_bytes = trn_rif_bytes;
+	}
+	else {
+		trn_rif_bytes = 0;
+		actual_rif_bytes = 0;
+	}
+
 	/* this is a silly hack for Linux 2.0.x. Read the comment below,
-	in front of the other #ifdef linux */
-	#ifdef linux
-	else if (frame_type == 1) {
+	in front of the other #ifdef linux. If we're sniffing our own NIC,
+	 we get a full RIF, sometimes with garbage */
+	if ((source_routed && trn_rif_bytes == 2 && frame_type == 1) ||
+		(!source_routed && frame_type == 1)) {
 		/* look for SNAP or IPX only */
 		if ( (pd[0x20] == 0xaa && pd[0x21] == 0xaa && pd[0x22] == 03) ||
 			 (pd[0x20] == 0xe0 && pd[0x21] == 0xe0) ) {
-			silly_linux = 1;
-			rif_bytes = 18;
+			actual_rif_bytes = 18;
 		}
 	}
-	#endif
-	offset += rif_bytes;
+	offset += actual_rif_bytes;
 
-	/* Make a copy of the src hwaddr, w/o source routing. I'll do this
-		for all packets, even non-sr packets */
-	memcpy(nonsr_hwaddr, &pd[8], 6);
-	nonsr_hwaddr[0] &= 127;
 
-	if (fd->win_info[0]) {
-		strcpy(fd->win_info[2], ether_to_str((guint8 *)&pd[2]));
-		strcpy(fd->win_info[1], ether_to_str(nonsr_hwaddr));
-		strcpy(fd->win_info[3], "TR");
-		sprintf(fd->win_info[4], "Token-Ring %s", fc[frame_type]);
+	/* information window */
+	if (fd->win_info[COL_NUM]) {
+		strcpy(fd->win_info[COL_DESTINATION], ether_to_str((guint8 *)&pd[2]));
+		strcpy(fd->win_info[COL_SOURCE], ether_to_str(trn_shost_nonsr));
+		strcpy(fd->win_info[COL_PROTOCOL], "TR");
+		sprintf(fd->win_info[COL_INFO], "Token-Ring %s", fc[frame_type]);
 	}
 
+	/* protocol analysis tree */
 	if (tree) {
-		ti = add_item_to_tree(GTK_WIDGET(tree), 0, 14 + rif_bytes,
-		  "Token-Ring (%d on wire, %d captured)", fd->pkt_len, fd->cap_len);
+		ti = add_item_to_tree(GTK_WIDGET(tree), 0, 14 + actual_rif_bytes,
+		  "Token-Ring");
 		fh_tree = gtk_tree_new();
-		gtk_tree_item_set_subtree(GTK_TREE_ITEM(ti), fh_tree);
+		add_subtree(ti, fh_tree, ETT_TOKEN_RING);
 		add_item_to_tree(fh_tree, 0, 1,
 			"Access Control: %s, Priority=%d, Monitor Count=%d, "
 			"Priority Reservation=%d",
-			((pd[0] & 16) >> 4) ? "Frame" : "Token",	/* frame/token */
-			((pd[0] & 224) >> 5),						/* priority */
-			((pd[0] & 8) >> 3),							/* monitor count */
-			((pd[0] & 7)));								/* priority reserv. */
+			((trn_ac & 16) >> 4) ? "Frame" : "Token",	/* frame/token */
+			((trn_ac & 224) >> 5),				/* priority */
+			((trn_ac & 8) >> 3),				/* monitor count */
+			((trn_ac & 7)));				/* priority reserv. */
 
 		add_item_to_tree(fh_tree, 1, 1,
 			"Frame Control: %s, Physical Control=%d (%s)",
-			fc[frame_type], (pd[1] & 15),
-			fc_pcf[(pd[1] & 15)]);
+			fc[frame_type], (trn_fc & 15),
+			fc_pcf[(trn_fc & 15)]);
 
 		add_item_to_tree(fh_tree, 2, 6, "Destination: %s",
-			ether_to_str((guint8 *) &pd[2]));
+			ether_to_str((guint8 *) trn_dhost));
 		add_item_to_tree(fh_tree, 8, 6, "Source: %s",
-			ether_to_str((guint8 *) &pd[8]));
+			ether_to_str((guint8 *) trn_shost));
 
 		if (source_routed) {
-			add_item_to_tree(fh_tree, 14, 1, "RIF length: %d bytes",
-				pd[14] & 31);
+			add_item_to_tree(fh_tree, 14, 1, "RIF length: %d bytes", trn_rif_bytes);
 
 			add_item_to_tree(fh_tree, 15, 1,
 				"%s, up to %d bytes in frame (LF=%d)",
@@ -161,7 +224,11 @@ dissect_tr(const u_char *pd, frame_data *fd, GtkTree *tree) {
 				rc_direction[(pd[15] & 128) >> 7],
 				rc_arrow[(pd[15] & 128) >> 7]);
 
-			add_ring_bridge_pairs(rif_bytes, pd, fh_tree);
+			/* if we have more than 2 bytes of RIF, then we have
+				ring/bridge pairs */
+			if (trn_rif_bytes > 2) {
+				add_ring_bridge_pairs(trn_rif_bytes, pd, fh_tree);
+			}
 		}
 
 		/* Linux 2.0.x has a problem in that the 802.5 code creates
@@ -173,20 +240,29 @@ dissect_tr(const u_char *pd, frame_data *fd, GtkTree *tree) {
 		to know the src hwaddr of the machine from which you were running
 		tcpdump. W/o that, however, I'm guessing that DSAP == SSAP if the
 		frame type is LLC.  It's very much a hack. -- Gilbert Ramirez */
-		#ifdef linux
-		else if (silly_linux) {
-			add_item_to_tree(fh_tree, 14, 18,
+		if (actual_rif_bytes > trn_rif_bytes) {
+			printf("trn_rif %d    actual_rif %d\n", trn_rif_bytes, actual_rif_bytes);
+			add_item_to_tree(fh_tree, 14 + trn_rif_bytes, actual_rif_bytes - trn_rif_bytes,
 				"Empty RIF from Linux 2.0.x driver. The sniffing NIC "
 				"is also running a protocol stack.");
 		}
-		#endif
+		/*
+		if (source_routed && (trn_rif_bytes == 2) && silly_linux) {
+			add_item_to_tree(fh_tree, 14 + trn_rif_bytes, 18 - actual_rif_bytes,
+				"Empty RIF from Linux 2.0.x driver. The sniffing NIC "
+				"is also running a protocol stack.");
+		}
+		else if ((!source_routed) && silly_linux ) {
+			add_item_to_tree(fh_tree, 14, 18,
+				"Empty RIF from Linux 2.0.x driver. The sniffing NIC "
+				"is also running a protocol stack.");
+		}*/
 	}
 
 	/* The package is either MAC or LLC */
 	switch (frame_type) {
 		/* MAC */
 		case 0:
-			/* dissect_trmac(pd, offset, fd, tree) */
 			dissect_trmac(pd, offset, fd, tree);
 			break;
 		case 1:
@@ -194,7 +270,7 @@ dissect_tr(const u_char *pd, frame_data *fd, GtkTree *tree) {
 			break;
 		default:
 			/* non-MAC, non-LLC, i.e., "Reserved" */
-			dissect_raw(pd, offset, fd, tree);
+			dissect_data(pd, offset, fd, tree);
 			break;
 	}
 }
