@@ -2,6 +2,8 @@
  * Routines for Token-Ring Media Access Control
  * Gilbert Ramirez <gram@verdict.uthscsa.edu>
  *
+ * $Id: packet-trmac.c,v 1.4 1998/10/10 03:32:15 gerald Exp $
+ *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@unicom.net>
  * Copyright 1998 Gerald Combs
@@ -22,7 +24,14 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#ifdef HAVE_SYS_TYPES_H
+# include <sys/types.h>
+#endif
+
 #include <pcap.h>
 
 #include <gtk/gtk.h>
@@ -30,15 +39,15 @@
 #include <stdio.h>
 
 #ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
+# include <sys/types.h>
 #endif
 
 #ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
+# include <netinet/in.h>
 #endif
 
-#include "packet.h"
 #include "ethereal.h"
+#include "packet.h"
 #include "etypes.h"
 
 struct vec_info {
@@ -81,10 +90,10 @@ mv_text(u_char cmd, int offset, frame_data *fd, GtkWidget *tree) {
 
 	while (mv[i].text != NULL) {
 		if (mv[i].cmd == cmd) {
-			if (fd->win_info[0]) {
+			if (fd->win_info[COL_NUM]) {
 				/* I can do this because no higher-level dissect()
 					will strcpy onto me. */
-				fd->win_info[4] = mv[i].text;
+				fd->win_info[COL_INFO] = mv[i].text;
 			}
 			if (tree) {
 				add_item_to_tree(tree, offset, 1, "Major Vector Command: %s",
@@ -227,7 +236,7 @@ sv_text(const u_char *pd, int pkt_offset, GtkWidget *tree)
 				"Isolating Error Counts (%d total)",
 				errors[0] + errors[1] + errors[2] + errors[3] + errors[4]);
 			sv_tree = gtk_tree_new();
-			gtk_tree_item_set_subtree(GTK_TREE_ITEM(ti), sv_tree);
+			add_subtree(ti, sv_tree, ETT_TR_IERR_CNT);
 
 			add_item_to_tree(sv_tree, pkt_offset+2, 1,
 				"Line Errors: %d", errors[0]);
@@ -248,7 +257,7 @@ sv_text(const u_char *pd, int pkt_offset, GtkWidget *tree)
 				"Non-Isolating Error Counts (%d total)",
 				errors[0] + errors[1] + errors[2] + errors[3] + errors[4]);
 			sv_tree = gtk_tree_new();
-			gtk_tree_item_set_subtree(GTK_TREE_ITEM(ti), sv_tree);
+			add_subtree(ti, sv_tree, ETT_TR_NERR_CNT);
 
 			add_item_to_tree(sv_tree, pkt_offset+2, 1,
 				"Lost Frame Errors: %d", errors[0]);
@@ -283,8 +292,8 @@ dissect_trmac(const u_char *pd, int offset, frame_data *fd, GtkTree *tree) {
 		"Configuration Report Server", "Ring Parameter Server",
 		"Ring Error Monitor" };
 
-	if (fd->win_info[0]) {
-		strcpy(fd->win_info[3], "TR MAC");
+	if (fd->win_info[COL_NUM]) {
+		strcpy(fd->win_info[COL_PROTOCOL], "TR MAC");
 	}
 
 	mv_length = ntohs(*((guint16*)&pd[offset]));
@@ -293,7 +302,7 @@ dissect_trmac(const u_char *pd, int offset, frame_data *fd, GtkTree *tree) {
 		ti = add_item_to_tree(GTK_WIDGET(tree), offset, mv_length,
 			"Media Access Control");
 		mac_tree = gtk_tree_new();
-		gtk_tree_item_set_subtree(GTK_TREE_ITEM(ti), mac_tree);
+		add_subtree(ti, mac_tree, ETT_TR_MAC);
 	}
 
 	/* Interpret the major vector */
