@@ -1,6 +1,8 @@
 /* packet-udp.c
  * Routines for UDP packet disassembly
  *
+ * $Id: packet-udp.c,v 1.4 1998/09/27 22:12:40 gerald Exp $
+ *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
  * Copyright 1998 Gerald Combs
@@ -21,22 +23,25 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
  
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include <gtk/gtk.h>
 
 #include <stdio.h>
 
 #ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
+# include <sys/types.h>
 #endif
 
 #ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
+# include <netinet/in.h>
 #endif
 
 #include "ethereal.h"
 #include "packet.h"
+#include "resolv.h"
 
 void
 dissect_udp(const u_char *pd, int offset, frame_data *fd, GtkTree *tree) {
@@ -51,19 +56,19 @@ dissect_udp(const u_char *pd, int offset, frame_data *fd, GtkTree *tree) {
   uh_ulen  = ntohs(uh->uh_ulen);
   uh_sum   = ntohs(uh->uh_sum);
   
-  if (fd->win_info[0]) {
-    strcpy(fd->win_info[3], "UDP");
-    sprintf(fd->win_info[4], "Source port: %d  Destination port: %d",
-      uh_sport, uh_dport);
+  if (fd->win_info[COL_NUM]) {
+    strcpy(fd->win_info[COL_PROTOCOL], "UDP");
+    sprintf(fd->win_info[COL_INFO], "Source port: %s  Destination port: %s",
+	    get_udp_port(uh_sport), get_udp_port(uh_dport));
   }
   
   if (tree) {
     ti = add_item_to_tree(GTK_WIDGET(tree), offset, 8,
       "User Datagram Protocol");
     udp_tree = gtk_tree_new();
-    gtk_tree_item_set_subtree(GTK_TREE_ITEM(ti), udp_tree);
-    add_item_to_tree(udp_tree, offset,     2, "Source port: %d", uh_sport);
-    add_item_to_tree(udp_tree, offset + 2, 2, "Destination port: %d", uh_dport);
+    add_subtree(ti, udp_tree, ETT_UDP);
+    add_item_to_tree(udp_tree, offset,     2, "Source port: %s", get_udp_port(uh_sport));
+    add_item_to_tree(udp_tree, offset + 2, 2, "Destination port: %s", get_udp_port(uh_dport));
     add_item_to_tree(udp_tree, offset + 4, 2, "Length: %d", uh_ulen);
     add_item_to_tree(udp_tree, offset + 6, 2, "Checksum: 0x%04x", uh_sum);
   }
@@ -73,10 +78,20 @@ dissect_udp(const u_char *pd, int offset, frame_data *fd, GtkTree *tree) {
 
   /* To do: make sure we aren't screwing ourselves with the MIN call. */
   switch (MIN(uh_sport, uh_dport)) {
-    case PORT_DNS:
+    case UDP_PORT_BOOTPS:
+      dissect_bootp(pd, offset, fd, tree);
+      break;
+    case UDP_PORT_DNS:
       dissect_dns(pd, offset, fd, tree);
       break;
+    case UDP_PORT_RIP:
+      /* we should check the source port too (RIP: UDP src and dst port 520) */
+      dissect_rip(pd, offset, fd, tree);
+      break;
+    case UDP_PORT_IPX: /* RFC 1234 */
+      dissect_ipx(pd, offset, fd, tree);
+      break;
     default:
-      dissect_raw(pd, offset, fd, tree);
+      dissect_data(pd, offset, fd, tree);
   }
 }
