@@ -1,6 +1,8 @@
 /* print.h
  * Definitions for printing packet analysis trees.
  *
+ * $Id: print.h,v 1.5 1998/10/10 03:32:20 gerald Exp $
+ *
  * Gilbert Ramirez <gram@verdict.uthscsa.edu>
  *
  * Ethereal - Network traffic analyzer
@@ -33,14 +35,16 @@ typedef struct pr_opts {
 	char	*cmd;
 
 	/* for the dialogue box */
-	GtkWidget	*window;
 	GtkWidget	*cmd_te;
 	GtkWidget	*file_te;
 } pr_opts;
 
 /* Functions in print.h */
 
-void printer_opts_cb(GtkWidget *, gpointer);
+GtkWidget *printer_prefs_show();
+void printer_prefs_ok(GtkWidget *w);
+void printer_prefs_cancel(GtkWidget *w);
+void print_tree(const u_char *pd, frame_data *fd, GtkTree *tree);
 void print_tree_text(FILE *fh, const u_char *pd, frame_data *fd, GtkTree *tree);
 void print_tree_ps(FILE *fh, const u_char *pd, frame_data *fd, GtkTree *tree);
 
