@@ -1,6 +1,8 @@
 /* util.c
  * Utility routines
  *
+ * $Id: util.c,v 1.4 1998/10/12 01:40:57 gerald Exp $
+ *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
  * Copyright 1998 Gerald Combs
@@ -21,14 +23,21 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include <glib.h>
 
 #include <gtk/gtk.h>
 
+#include <stdarg.h>
 #include <strings.h>
 
 #include "util.h"
 
+#include "image/icon-excl.xpm"
+
 const gchar *bm_key = "button mask";
 
 /* Simple dialog function - Displays a dialog box with the supplied message
@@ -44,21 +53,24 @@ const gchar *bm_key = "button mask";
  * To do:
  * - Switch to variable args
  */
+ 
+#define ESD_MAX_MSG_LEN 1024
 void
-simple_dialog(gint type, gint *btn_mask, gchar *message) {
-  GtkWidget *win, *main_vb, *top_hb, *type_pm, *msg_label,
-            *bbox, *ok_btn, *cancel_btn;
-  GdkPixmap *pixmap;
-  GdkBitmap *mask;
-  GtkStyle  *style;
-
+simple_dialog(gint type, gint *btn_mask, gchar *msg_format, ...) {
+  GtkWidget   *win, *main_vb, *top_hb, *type_pm, *msg_label,
+              *bbox, *ok_btn, *cancel_btn;
+  GdkPixmap   *pixmap;
+  GdkBitmap   *mask;
+  GtkStyle    *style;
+  GdkColormap *cmap;
+  va_list      ap;
+  gchar        message[ESD_MAX_MSG_LEN];
+  
   /* Main window */
   win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_container_border_width(GTK_CONTAINER(win), 7);
   gtk_window_set_title(GTK_WINDOW(win), "Ethereal: Warning");
   gtk_object_set_data(GTK_OBJECT(win), bm_key, btn_mask);
-  gtk_widget_show(win);
-
 
   /* Container for our rows */
   main_vb = gtk_vbox_new(FALSE, 5);
@@ -72,12 +84,17 @@ simple_dialog(gint type, gint *btn_mask, gchar *message) {
   gtk_widget_show(top_hb);
   
   style = gtk_widget_get_style(win);
-  pixmap = gdk_pixmap_create_from_xpm_d(win->window,  &mask,
+  cmap  = gdk_colormap_get_system();
+  pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL, cmap,  &mask,
     &style->bg[GTK_STATE_NORMAL], icon_excl_xpm);
   type_pm = gtk_pixmap_new(pixmap, mask);
   gtk_container_add(GTK_CONTAINER(top_hb), type_pm);
   gtk_widget_show(type_pm);
 
+  /* Load our vararg list into the message string */
+  va_start(ap, msg_format);
+  vsnprintf(message, ESD_MAX_MSG_LEN, msg_format, ap);
+
   msg_label = gtk_label_new(message);
   gtk_label_set_justify(GTK_LABEL(msg_label), GTK_JUSTIFY_FILL);
   gtk_container_add(GTK_CONTAINER(top_hb), msg_label);
@@ -102,11 +119,14 @@ simple_dialog(gint type, gint *btn_mask, gchar *message) {
     gtk_signal_connect(GTK_OBJECT(cancel_btn), "clicked",
       GTK_SIGNAL_FUNC(simple_dialog_cancel_cb), (gpointer) win);
     gtk_container_add(GTK_CONTAINER(bbox), cancel_btn);
+    GTK_WIDGET_SET_FLAGS(cancel_btn, GTK_CAN_DEFAULT);
     gtk_widget_show(cancel_btn);
   }
 
   if (btn_mask)
     *btn_mask = ESD_BTN_OK;
+
+  gtk_widget_show(win);
 }
 
 void
